class SitefResponse(object):
    def __init__(self):
        # type: (...) -> None

        self.nsu = None
        self.nsu_host = None
        self.nsu_origin = None
        self.transaction_value = None
        self.customer_receipt = None
        self.merchant_receipt = None
        self.transaction_identifier = None

