from ._AlignCommandProcessor import AlignCommandProcessor
from ._BoldCommandProcessor import BoldCommandProcessor
from ._RepeatCommandProcessor import RepeatCommandProcessor
from ._FontCommandProcessor import FontCommandProcessor
from ._NewLineCommandProcessor import NewLineCommandProcessor
from ._CutPaperCommandProcessor import CutPaperCommandProcessor
