from helper import ExtendedEnum
from messagebus import (
    TokenCreator,
    TokenPriority,
)

MESSAGE_GROUP = "114"


class ListenedTokens(ExtendedEnum):
    TK_ALERTS_GET_CONFIGURED_ALERTS = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "1")
    TK_ALERTS_UPDATE_ALERTS = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "2")
