from datetime import datetime

from typing import Optional


class DeviceStatusDto(object):
    def __init__(
        self,
        type,                       # type: str
        name,                       # type: str
        enabled,                    # type: bool
        last_update=None,           # type: Optional[datetime]
        last_update_operator=None,  # type: Optional[str]
    ):
        # type: (...) -> None

        self.type = type
        self.name = name
        self.enabled = enabled
        self.last_update = last_update
        self.last_update_operator = last_update_operator
