class Payment:
    def __init__(
        self,
        payment_id,         # type: int
        name,               # type: str
        currency,           # type: str
        change_limit,       # type: float
        electronic_type,    # type: int
        open_drawer,        # type: bool
        need_auth,          # type: int
        parent_id,          # type: int
        custom_attr,        # type: str
    ):
        # type: (...) -> None

        self.payment_id = payment_id
        self.name = name
        self.currency = currency
        self.change_limit = change_limit
        self.electronic_type = electronic_type
        self.open_drawer = open_drawer
        self.need_auth = need_auth
        self.parent_id = parent_id
        self.custom_attr = custom_attr
