from typing import Optional

from application.service import MediaUpdaterService


class MediaUpdaterInteractor(object):
    def __init__(
        self,
        media_updater_service,  # type: MediaUpdaterService
    ):
        # type: (...) -> None

        self.media_updater_service = media_updater_service

    def update_media(
        self,
        force_update=False  # type: Optional[bool]
    ):
        # type: (bool) -> None

        return self.media_updater_service.update_media(force_update=force_update)
