# -*- coding: utf-8 -*-
import os
import component
from helper import (
    config_logger,
    import_pydevd,
)


def main():
    import_pydevd(os.environ["LOADERCFG"], 9134)
    config_logger(os.environ["LOADERCFG"], "TableManager", max_files=25)

    component.main()
