data = (
'bols',    # 0x00
'bolt',    # 0x01
'bolp',    # 0x02
'bolh',    # 0x03
'bom',    # 0x04
'bob',    # 0x05
'bobs',    # 0x06
'bos',    # 0x07
'boss',    # 0x08
'bong',    # 0x09
'boj',    # 0x0a
'boc',    # 0x0b
'bok',    # 0x0c
'bot',    # 0x0d
'bop',    # 0x0e
'boh',    # 0x0f
'bwa',    # 0x10
'bwag',    # 0x11
'bwagg',    # 0x12
'bwags',    # 0x13
'bwan',    # 0x14
'bwanj',    # 0x15
'bwanh',    # 0x16
'bwad',    # 0x17
'bwal',    # 0x18
'bwalg',    # 0x19
'bwalm',    # 0x1a
'bwalb',    # 0x1b
'bwals',    # 0x1c
'bwalt',    # 0x1d
'bwalp',    # 0x1e
'bwalh',    # 0x1f
'bwam',    # 0x20
'bwab',    # 0x21
'bwabs',    # 0x22
'bwas',    # 0x23
'bwass',    # 0x24
'bwang',    # 0x25
'bwaj',    # 0x26
'bwac',    # 0x27
'bwak',    # 0x28
'bwat',    # 0x29
'bwap',    # 0x2a
'bwah',    # 0x2b
'bwae',    # 0x2c
'bwaeg',    # 0x2d
'bwaegg',    # 0x2e
'bwaegs',    # 0x2f
'bwaen',    # 0x30
'bwaenj',    # 0x31
'bwaenh',    # 0x32
'bwaed',    # 0x33
'bwael',    # 0x34
'bwaelg',    # 0x35
'bwaelm',    # 0x36
'bwaelb',    # 0x37
'bwaels',    # 0x38
'bwaelt',    # 0x39
'bwaelp',    # 0x3a
'bwaelh',    # 0x3b
'bwaem',    # 0x3c
'bwaeb',    # 0x3d
'bwaebs',    # 0x3e
'bwaes',    # 0x3f
'bwaess',    # 0x40
'bwaeng',    # 0x41
'bwaej',    # 0x42
'bwaec',    # 0x43
'bwaek',    # 0x44
'bwaet',    # 0x45
'bwaep',    # 0x46
'bwaeh',    # 0x47
'boe',    # 0x48
'boeg',    # 0x49
'boegg',    # 0x4a
'boegs',    # 0x4b
'boen',    # 0x4c
'boenj',    # 0x4d
'boenh',    # 0x4e
'boed',    # 0x4f
'boel',    # 0x50
'boelg',    # 0x51
'boelm',    # 0x52
'boelb',    # 0x53
'boels',    # 0x54
'boelt',    # 0x55
'boelp',    # 0x56
'boelh',    # 0x57
'boem',    # 0x58
'boeb',    # 0x59
'boebs',    # 0x5a
'boes',    # 0x5b
'boess',    # 0x5c
'boeng',    # 0x5d
'boej',    # 0x5e
'boec',    # 0x5f
'boek',    # 0x60
'boet',    # 0x61
'boep',    # 0x62
'boeh',    # 0x63
'byo',    # 0x64
'byog',    # 0x65
'byogg',    # 0x66
'byogs',    # 0x67
'byon',    # 0x68
'byonj',    # 0x69
'byonh',    # 0x6a
'byod',    # 0x6b
'byol',    # 0x6c
'byolg',    # 0x6d
'byolm',    # 0x6e
'byolb',    # 0x6f
'byols',    # 0x70
'byolt',    # 0x71
'byolp',    # 0x72
'byolh',    # 0x73
'byom',    # 0x74
'byob',    # 0x75
'byobs',    # 0x76
'byos',    # 0x77
'byoss',    # 0x78
'byong',    # 0x79
'byoj',    # 0x7a
'byoc',    # 0x7b
'byok',    # 0x7c
'byot',    # 0x7d
'byop',    # 0x7e
'byoh',    # 0x7f
'bu',    # 0x80
'bug',    # 0x81
'bugg',    # 0x82
'bugs',    # 0x83
'bun',    # 0x84
'bunj',    # 0x85
'bunh',    # 0x86
'bud',    # 0x87
'bul',    # 0x88
'bulg',    # 0x89
'bulm',    # 0x8a
'bulb',    # 0x8b
'buls',    # 0x8c
'bult',    # 0x8d
'bulp',    # 0x8e
'bulh',    # 0x8f
'bum',    # 0x90
'bub',    # 0x91
'bubs',    # 0x92
'bus',    # 0x93
'buss',    # 0x94
'bung',    # 0x95
'buj',    # 0x96
'buc',    # 0x97
'buk',    # 0x98
'but',    # 0x99
'bup',    # 0x9a
'buh',    # 0x9b
'bweo',    # 0x9c
'bweog',    # 0x9d
'bweogg',    # 0x9e
'bweogs',    # 0x9f
'bweon',    # 0xa0
'bweonj',    # 0xa1
'bweonh',    # 0xa2
'bweod',    # 0xa3
'bweol',    # 0xa4
'bweolg',    # 0xa5
'bweolm',    # 0xa6
'bweolb',    # 0xa7
'bweols',    # 0xa8
'bweolt',    # 0xa9
'bweolp',    # 0xaa
'bweolh',    # 0xab
'bweom',    # 0xac
'bweob',    # 0xad
'bweobs',    # 0xae
'bweos',    # 0xaf
'bweoss',    # 0xb0
'bweong',    # 0xb1
'bweoj',    # 0xb2
'bweoc',    # 0xb3
'bweok',    # 0xb4
'bweot',    # 0xb5
'bweop',    # 0xb6
'bweoh',    # 0xb7
'bwe',    # 0xb8
'bweg',    # 0xb9
'bwegg',    # 0xba
'bwegs',    # 0xbb
'bwen',    # 0xbc
'bwenj',    # 0xbd
'bwenh',    # 0xbe
'bwed',    # 0xbf
'bwel',    # 0xc0
'bwelg',    # 0xc1
'bwelm',    # 0xc2
'bwelb',    # 0xc3
'bwels',    # 0xc4
'bwelt',    # 0xc5
'bwelp',    # 0xc6
'bwelh',    # 0xc7
'bwem',    # 0xc8
'bweb',    # 0xc9
'bwebs',    # 0xca
'bwes',    # 0xcb
'bwess',    # 0xcc
'bweng',    # 0xcd
'bwej',    # 0xce
'bwec',    # 0xcf
'bwek',    # 0xd0
'bwet',    # 0xd1
'bwep',    # 0xd2
'bweh',    # 0xd3
'bwi',    # 0xd4
'bwig',    # 0xd5
'bwigg',    # 0xd6
'bwigs',    # 0xd7
'bwin',    # 0xd8
'bwinj',    # 0xd9
'bwinh',    # 0xda
'bwid',    # 0xdb
'bwil',    # 0xdc
'bwilg',    # 0xdd
'bwilm',    # 0xde
'bwilb',    # 0xdf
'bwils',    # 0xe0
'bwilt',    # 0xe1
'bwilp',    # 0xe2
'bwilh',    # 0xe3
'bwim',    # 0xe4
'bwib',    # 0xe5
'bwibs',    # 0xe6
'bwis',    # 0xe7
'bwiss',    # 0xe8
'bwing',    # 0xe9
'bwij',    # 0xea
'bwic',    # 0xeb
'bwik',    # 0xec
'bwit',    # 0xed
'bwip',    # 0xee
'bwih',    # 0xef
'byu',    # 0xf0
'byug',    # 0xf1
'byugg',    # 0xf2
'byugs',    # 0xf3
'byun',    # 0xf4
'byunj',    # 0xf5
'byunh',    # 0xf6
'byud',    # 0xf7
'byul',    # 0xf8
'byulg',    # 0xf9
'byulm',    # 0xfa
'byulb',    # 0xfb
'byuls',    # 0xfc
'byult',    # 0xfd
'byulp',    # 0xfe
'byulh',    # 0xff
)
